# vim: set ts=8 sw=4 sts=4 et ai tw=79:
"""
The Exact Online REST API Library in Python (EORALP), licensed under the
LGPLv3+.
Copyright (C) 2015 Walter Doekes, OSSO B.V.

See README.rst for more info.
"""
